require 'openssl'
require 'base64'
require 'json'

module Redsys
  class Tpv
    attr_accessor :amount, :language, :order, :currency, :merchant_code, :terminal,
                  :transaction_type, :merchant_url, :url_ok, :url_ko, :sha1, :signature

    def self.tpv_url
      Rails.configuration.redsys_rails[:url]
    end

    def self.signature_version
      Rails.configuration.redsys_rails[:signature_version]
    end

    def initialize(amount, order, language, merchant_url = nil, url_ok = nil, url_ko = nil)
      amount        ||= 0
      order         ||= 0
      language      ||= '001'
      merchant_url  ||= ''
      url_ok        ||= ''
      url_ko        ||= ''

      @amount = (amount * 100).to_i.to_s
      @order = order.to_s.rjust(4, '0')
      @language = language
      @merchant_url = merchant_url
      @url_ok = url_ok
      @url_ko = url_ko

      @currency = Rails.configuration.redsys_rails[:merchant_currency]
      @merchant_code = Rails.configuration.redsys_rails[:merchant_code]
      @terminal = Rails.configuration.redsys_rails[:merchant_terminal]
      @transaction_type = Rails.configuration.redsys_rails[:merchant_transaction_type]
    end

    def merchant_params
      "#{Base64.strict_encode64(merchant_params_json)}"
    end

    def merchant_params_json
      merchant_parameters = { 
        :DS_MERCHANT_AMOUNT => @amount,
        :DS_MERCHANT_ORDER => @order,
        :DS_MERCHANT_MERCHANTCODE => @merchant_code,
        :DS_MERCHANT_CURRENCY => @currency,
        :DS_MERCHANT_TRANSACTIONTYPE => @transaction_type,
        :DS_MERCHANT_TERMINAL => @terminal,
        :DS_MERCHANT_MERCHANTURL => @merchant_url,
        :DS_MERCHANT_CONSUMERLANGUAGE => @language,
        :DS_MERCHANT_URLOK => @url_ok,
        :DS_MERCHANT_URLKO => @url_ko 
      }
      JSON.generate(merchant_parameters)
    end

    def merchant_signature_3des
      Base64.strict_encode64(encrypt_3DES(@order, Base64.strict_decode64(Rails.configuration.redsys_rails[:sha_256_key])))
    end

    def merchant_signature
      key = Base64.strict_decode64(Rails.configuration.redsys_rails[:sha_256_key])
      key = encrypt_3DES(@order, key)
      encrypt_mac256(merchant_params, key)
    end

    private

      def encrypt_mac256(data, key)
        Base64.strict_encode64(OpenSSL::HMAC.digest(OpenSSL::Digest.new('sha256'), key, data))
      end
    
      def encrypt_3DES(data, key)
        cipher = OpenSSL::Cipher::Cipher.new('DES3')
        cipher.encrypt
        cipher.key = key
        cipher.padding = 0
        block_length = 8
        data_str = data
        data_str += "\0" until data_str.bytesize % block_length == 0
        output = cipher.update(data_str)
        output << cipher.final
        output
      end
  end
end